@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	$Id: mhamain.pl,v 2.71 2003/08/13 03:03:17 ehood Exp $
+##	$Id: mhamain.pl,v 2.93 2006/06/10 02:42:58 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
 ##	Main library for MHonArc.
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-2003	Earl Hood, mhonarc@mhonarc.org
+##    Copyright (C) 1995-2005	Earl Hood, mhonarc@mhonarc.org
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -29,10 +29,10 @@ package mhonarc;
 
 require 5;
 
-$VERSION = '2.6.8';
+$VERSION = '2.6.16';
 $VINFO =<<EndOfInfo;
   MHonArc v$VERSION (Perl $] $^O)
-  Copyright (C) 1995-2003  Earl Hood, mhonarc\@mhonarc.org
+  Copyright (C) 1995-2005  Earl Hood, mhonarc\@mhonarc.org
   MHonArc comes with ABSOLUTELY NO WARRANTY and MHonArc may be copied only
   under the terms of the GNU General Public License, which may be found in
   the MHonArc distribution.
@@ -265,6 +265,7 @@ sub doit {
 	    $MULTIIDX   = 0;
 	    $IdxPageNum = 1;
 	    $NumOfPages = 1;
+	    $NumOfPrintedPages = 1;
 	}
 	if ($THREAD) {
 	    compute_threads();
@@ -295,8 +296,18 @@ sub doit {
 	($index, $fields) = read_mail_header($handle);
 
 	if ($index) {
-	    $AddIndex{$index} = 1;
-	    read_mail_body($handle, $index, $fields, $NoMsgPgs);
+	    if (defined(read_mail_body(
+		    $handle, $index, $fields, $NoMsgPgs))) {
+		$AddIndex{$index} = 1;
+		## Invoke callback if defined
+		if (defined($CBMessageConverted) &&
+			defined(&$CBMessageConverted)) {
+		    &$CBMessageConverted($fields, +{
+			folder => undef,
+			file   => '-',
+		    });
+		}
+	    }
 	}
 
     ## Adding/converting mail{boxes,folders}
@@ -321,7 +332,7 @@ sub doit {
 
 		local($_);
 		MHFILE: foreach (@files) {
-		    $mesgfile = "${mbox}${DIRSEP}${_}";
+		    $mesgfile = join($DIRSEP, $mbox, $_);
 		    eval {
 			$fh = file_open($mesgfile);
 		    };
@@ -335,17 +346,30 @@ sub doit {
 
 		    #  Process message if valid
 		    if ($index) {
-			if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
-			read_mail_body($fh, $index, $fields, $NoMsgPgs);
+			if (defined(read_mail_body(
+				$fh, $index, $fields, $NoMsgPgs))) {
+			    if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
 
-			#  Check if conserving memory
-			if ($SLOW && $DoArchive) {
-			    output_mail($index, 1, 1);
-			    $Update{$IndexNum{$index}} = 1;
-			}
-			if ($SLOW || !$DoArchive) {
-			    delete $MsgHead{$index};
-			    delete $Message{$index};
+			    #  Check if conserving memory
+			    if ($SLOW && $DoArchive) {
+				output_mail($index, 1, 1);
+				$Update{$IndexNum{$index}} = 1;
+			    }
+			    if ($SLOW || !$DoArchive) {
+				delete $MsgHead{$index};
+				delete $Message{$index};
+			    }
+			    ## Invoke callback if defined
+			    if (defined($CBMessageConverted) &&
+				    defined(&$CBMessageConverted)) {
+				&$CBMessageConverted($fields, +{
+				    folder => $mbox,
+				    file   => $mesgfile,
+				});
+			    }
+
+			} else {
+			    $index = undef;
 			}
 		    }
 		    close($fh);
@@ -374,25 +398,45 @@ sub doit {
 		    ($index, $fields) = read_mail_header($fh);
 
 		    if ($index) {
-			if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
-			read_mail_body($fh, $index, $fields, $NoMsgPgs);
+			if (defined(read_mail_body(
+				$fh, $index, $fields, $NoMsgPgs))) {
+			    if ($ADD && !$SLOW) { $AddIndex{$index} = 1; }
 
-			if ($SLOW && $DoArchive) {
-			    output_mail($index, 1, 1);
-			    $Update{$IndexNum{$index}} = 1;
-			}
-			if ($SLOW || !$DoArchive) {
-			    delete $MsgHead{$index};
-			    delete $Message{$index};
+			    if ($SLOW && $DoArchive) {
+				output_mail($index, 1, 1);
+				$Update{$IndexNum{$index}} = 1;
+			    }
+			    if ($SLOW || !$DoArchive) {
+				delete $MsgHead{$index};
+				delete $Message{$index};
+			    }
+			    ## Invoke callback if defined
+			    if (defined($CBMessageConverted) &&
+				    defined(&$CBMessageConverted)) {
+				&$CBMessageConverted($fields, +{
+				    folder => $mbox,
+				    file   => undef,
+				});
+			    }
+			} else {
+			    $index = undef;
 			}
 
 		    } else {
+			# skip passed message body
 			read_mail_body($fh, $index, $fields, 1);
 		    }
 		}
 		close($fh);
 
 	    } # END: else UUCP mailbox
+
+	    ## Invoke callback if defined
+	    if (defined($CBMailFolderRead) &&
+		    defined(&$CBMailFolderRead)) {
+		&$CBMailFolderRead($mbox);
+	    }
+
 	} # END: foreach $mbox
     } # END: Else converting mailboxes
     print "\n"  unless $QUIET;
@@ -678,9 +722,17 @@ sub compute_page_total {
 	$NumOfPages   = int($NumOfMsgs/$IDXSIZE);
 	++$NumOfPages      if ($NumOfMsgs/$IDXSIZE) > $NumOfPages;
 	$NumOfPages   = 1  if $NumOfPages == 0;
+
+	$NumOfPrintedPages = $NumOfPages;
+	if (($MAXPGS > 0) && ($MAXPGS < $NumOfPages)) {
+	    $NumOfPrintedPages = $MAXPGS;
+	}
+
     } else {
 	$NumOfPages = 1;
+	$NumOfPrintedPages = 1;
     }
+
 }
 
 ##---------------------------------------------------------------------------
@@ -831,9 +883,13 @@ sub read_mail_header {
     ##-------------##
     ## Get Subject ##
     ##-------------##
-    if (defined($fields->{'subject'}) && ($fields->{'subject'}[0] =~ /\S/)) {
+    if (defined($fields->{'subject'})) {
 	($sub = $fields->{'subject'}[0]) =~ s/\s+$//;
-	$sub = subject_strip($sub)  if $SubStripCode;
+	if ($SubStripCode) {
+	    $fields->{'x-mha-org-subject'} = $sub;
+	    $sub = subject_strip($sub);
+	}
+	$fields->{'subject'} = [ $sub ];    # Make sure only one subject
     } else {
 	$sub = '';
     }
@@ -976,7 +1032,11 @@ sub read_mail_body {
 
     ## Invoke callback if defined
     if (defined($CBRawMessageBodyRead) && defined(&$CBRawMessageBodyRead)) {
-	&$CBRawMessageBodyRead($fields, \$data);
+	if (!&$CBRawMessageBodyRead($fields, \$data)) {
+	    # reverse effect of read_mail_header()
+	    delmsg_from_hashes($index);
+	    return undef;
+	}
     }
 
     ## Define "globals" for use by filters
@@ -993,7 +1053,11 @@ sub read_mail_body {
 
     ## Invoke callback if defined
     if (defined($CBMessageBodyRead) && defined(&$CBMessageBodyRead)) {
-	&$CBMessageBodyRead($fields, \$ret, \@files);
+	if (!&$CBMessageBodyRead($fields, \$ret, \@files)) {
+	    # reverse effect of read_mail_header()
+	    delmsg_from_hashes($index);
+	    return undef;
+	}
 	$Message{$index} = $ret;
     } else {
 	$Message{$index} = $ret;
@@ -1004,7 +1068,7 @@ sub read_mail_body {
 	     qq/Warning: Empty body data generated:\n/,
 	     qq/         Message-Id: $MHAmsgid\n/,
 	     qq/         Message Number: $MHAmsgnum\n/,
-	     qq/         Content-Type/,
+	     qq/         Content-Type: /,
 			 ($fields->{'content-type'}[0] || 'text/plain'),
 			 qq/\n/;
 	$ret = '';
@@ -1084,25 +1148,31 @@ sub output_mail {
 
 	# Output comments -- more informative, but can be used for
 	#		     error recovering.
-	print $msghandle 
-	    "<!-- ", commentize("MHonArc v$VERSION"), " -->\n",
-	    "<!--X-Subject: ",      commentize($Subject{$index}), " -->\n",
-	    "<!--X-From-R13: ",	    commentize(mrot13($From{$index})), " -->\n",
-	    "<!--X-Date: ", 	    commentize($Date{$index}), " -->\n",
-	    "<!--X-Message-Id: ",   commentize($Index2MsgId{$index}), " -->\n",
-	    "<!--X-Content-Type: ", commentize($ContentType{$index}), " -->\n";
-		  #ContentType
+	print $msghandle "<!-- ", commentize("MHonArc v$VERSION"), " -->\n";
+	if ($PrintXComments) {
+	    print $msghandle 
+		"<!--X-Subject: ",
+		    commentize($Subject{$index}), " -->\n",
+		"<!--X-From-R13: ",
+		    commentize(mrot13($From{$index})), " -->\n",
+		"<!--X-Date: ",
+		    commentize($Date{$index}), " -->\n",
+		"<!--X-Message-Id: ",
+		    commentize($Index2MsgId{$index}), " -->\n",
+		"<!--X-Content-Type: ",
+		    commentize($ContentType{$index}), " -->\n";
 
-	if (defined($Refs{$index})) {
-	    foreach (@{$Refs{$index}}) {
-		print $msghandle
-		    "<!--X-Reference: ", commentize($_), " -->\n";
-			  #Reference-Id
+	    if (defined($Refs{$index})) {
+		foreach (@{$Refs{$index}}) {
+		    print $msghandle
+			"<!--X-Reference: ", commentize($_), " -->\n";
+			      #Reference-Id
+		}
 	    }
-	}
-	if (defined($Derived{$index})) {
-	    foreach (@{$Derived{$index}}) {
-		print $msghandle "<!--X-Derived: ", commentize($_), " -->\n";
+	    if (defined($Derived{$index})) {
+		foreach (@{$Derived{$index}}) {
+		    print $msghandle "<!--X-Derived: ", commentize($_), " -->\n";
+		}
 	    }
 	}
 	print $msghandle "<!--X-Head-End-->\n";
@@ -1350,15 +1420,44 @@ sub output_mail {
 #############################################################################
 
 ##---------------------------------------------------------------------------
-##	delmsg delets a message from the archive.
+##	delmsg deletes a message from the archive.
 ##
 sub delmsg {
     my $key = shift;
-    my($pathname);
+    my($filename, $derived) = delmsg_from_hashes($key);
+    return 0  unless defined($filename);
+
+    if (!$KeepOnRmm) {
+	file_remove($filename);
+	if (defined($derived)) {
+	    my $pathname;
+	    foreach $filename (@{$derived}) {
+		$pathname = (OSis_absolute_path($filename)) ?
+				$filename : join($DIRSEP, $OUTDIR, $filename);
+		if (-d $pathname) {
+		    dir_remove($pathname);
+		} else {
+		    file_remove($pathname);
+		}
+	    }
+	}
+    }
+    1;
+}
+
+##---------------------------------------------------------------------------
+##	delmsg_from_hashes deletes all message info from db hashes.
+##	Return value is a list of two items: pathname to message
+##	file and array ref to any derived files.
+##
+sub delmsg_from_hashes {
+    my $key = shift;
 
     #&defineIndex2MsgId();
-    my $msgnum = $IndexNum{$key};  return 0  if ($msgnum eq '');
+    my $msgnum = $IndexNum{$key};
+    return (undef, undef)  if ($msgnum eq '');
     my $filename = join($DIRSEP, $OUTDIR, &msgnum_filename($msgnum));
+
     delete $ContentType{$key};
     delete $Date{$key};
     delete $From{$key};
@@ -1366,19 +1465,12 @@ sub delmsg {
     delete $Refs{$key};
     delete $Subject{$key};
     delete $MsgId{$Index2MsgId{$key}};
-    file_remove($filename)  unless $KeepOnRmm;
-    foreach $filename (@{$Derived{$key}}) {
-	$pathname = (OSis_absolute_path($filename)) ?
-			$filename : join($DIRSEP, $OUTDIR, $filename);
-	if (-d $pathname) {
-	    dir_remove($pathname)  unless $KeepOnRmm;
-	} else {
-	    file_remove($pathname)  unless $KeepOnRmm;
-	}
-    }
+
+    my $derived = $Derived{$key};
     delete $Derived{$key};
+
     $NumOfMsgs--;
-    1;
+    ($filename, $derived);
 }
 
 ##---------------------------------------------------------------------------
