@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	$Id: mhtxtplain.pl,v 2.40 2003/08/02 20:38:14 ehood Exp $
+##	$Id: mhtxtplain.pl,v 2.47 2005/06/20 04:25:18 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
@@ -12,7 +12,7 @@
 ##              </MIMEFILTERS>
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-2002	Earl Hood, mhonarc@mhonarc.org
+##    Copyright (C) 1995-2005	Earl Hood, mhonarc@mhonarc.org
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -39,18 +39,18 @@ sub Q_SIMPLE() { 1; }
 sub Q_FANCY()  { 2; }
 sub Q_FLOWED() { 3; }
 
-$Url    	= '(http://|https://|ftp://|afs://|wais://|telnet://|ldap://' .
-		   '|gopher://|news:|nntp:|mid:|cid:|mailto:|prospero:)';
-$UrlExp 	= $Url . q/[^\s\(\)\|<>"'\0-\037]+/ .
+$UrlExp 	= $readmail::UrlRxStr .
+			 q/[^\s\(\)\|<>"'\0-\037]+/ .
 			 q/[^\.?!;,"'\|\[\]\(\)\s<>\0-\037]/;
-$HUrlExp        = $Url . q/(?:&(?![gl]t;)|[^\s\(\)\|<>"'\&\0-\037])+/ .
+$HUrlExp        = $readmail::UrlRxStr .
+			 q/(?:&(?![gl]t;)|[^\s\(\)\|<>"'\&\0-\037])+/ .
 			 q/[^\.?!;,"'\|\[\]\(\)\s<>\&\0-\037]/;
 $QuoteChars	= '[>]';
 $HQuoteChars	= '&gt;';
 
 $StartFlowedQuote =
-  '<blockquote style="border-left: #0000FF solid 0.1em; '.
-                     'margin: 0em; padding-left: 1.0em">';
+  '<blockquote style="border-left: #5555EE solid 0.2em; '.
+                     'margin: 0em; padding-left: 0.85em">';
 $EndFlowedQuote   = "</blockquote>";
 $StartFixedQuote  = '<pre style="margin: 0em;">';
 $EndFixedQuote    = '</pre>';
@@ -90,7 +90,16 @@ $EndFixedQuote    = '</pre>';
 ##
 ##	keepspace	Preserve whitespace if nonfixed
 ##
-##	nourl		Do hyperlink URLs
+##	link="scheme1,scheme2,..."
+##			A comma separate list of URL schemes to hyperlink.
+##			Only URL with the given schemes will be linked.
+##
+##	nolink="scheme1,scheme2,..."
+##			A comma separate list of URL schemes to not
+##			hyperlink.  URLs with the given scheme will not
+##			converted into hyperlinks.
+##
+##	nourl		Do not hyperlink URLs.
 ##
 ##	nonfixed	Use normal typeface
 ##
@@ -197,8 +206,11 @@ sub filter {
 		} else {
 		    push(@files,
 			 mhonarc::write_attachment(
-			    'application/octet-stream', \$uddata, $atdir,
-			    ($usename?$file:''), $inext));
+			    'application/octet-stream', \$uddata, {
+				'-dirpath'  => $atdir,
+				'-filename' => ($usename?$file:''),
+				'-ext'      => $inext
+			    }));
 		    $urlfile = mhonarc::htmlize($files[$#files]);
 
 		    # create link to file
@@ -305,7 +317,7 @@ sub filter {
 
     ## Check if max-width set
     if (($maxwidth > 0) && ($quote_style != Q_FLOWED)) {
-	$$data =~ s/^(.*)$/&break_line($1, $maxwidth)/gem;
+	break_lines($data, $charset, $maxwidth);
     }
 
     ## Convert data according to charset
@@ -333,8 +345,16 @@ sub filter {
     # v5.8.0).  Hence, code changed to use m//g and substr(), which
     # appears to avoid perl crashing (ehood, Dec 2002).
     #
-    # Initial code for format=flowed contributed by Ken Hirsch (May 2002).
-    # text/plain; format=flowed defined in RFC2646
+    # To fix bug #12512, flowed code changed to process each quote
+    # chunk line-by-line instead of as one entity.  The reason is
+    # that RFC-2646 does not define a "paragraph" by two consective
+    # CRLF sequences but by flowed vs non-flowed, which can occur
+    # with no blank lines in between (ehood, May 2005).
+    #
+    # Initial code for format=flowed contributed by Ken Hirsch (May 2002),
+    # but it has drastically changes since then.
+    #
+    # text/plain; format=flowed defined in RFC-2646
 
     if ($quote_style == Q_FLOWED) {
 	my($chunk, $qd, $offset);
@@ -375,72 +395,85 @@ sub filter {
 	    }
 	    $chunk =~ s/^-- $/--/mg; # special case for '-- '
 
-	    my @paras = split(/(\n\n)/, $chunk);
-	    my $para;
-	    $chunk = '';
-	    foreach $para (@paras) {
-		if ($para =~ /\A\n+\Z/) {
-		    $chunk .= "<br>\n" x length($para);
-		    next;
-		}
-		$para =~ s/^\n/<br>/;
-		my $nls = ($para =~ tr/\n/\n/);
-		if (($para =~ / \n/) || ($para =~ / \Z/) ||
-			($nls < 1) ||
-			(($nls == 1) && ($para =~ /\S/)
-			 && ($para =~ /\n\Z/))) {
-		    # flowed format
-		    $para =~ s/^(|.*[^ ])(\n)(?!\Z)/
-			       ($keepspace ? &preserve_space($1) : $1) .
-			       '<br>'.$2/mgex;
-		    if ($nonfixed) {
-			$chunk .= $para;
-		    } else {
-			$chunk .= '<tt>'.$para.'</tt>';
+	    # Parse chunk line at a time to determine how it is rendered.
+	    my $new_chunk = "";
+	    my $line      = "";
+	    my $inflow    = 0;
+	    my $infixed   = 0;
+	    FLOWED_LINE: while ($chunk ne "") {
+		# Grab next line: Pattern should always match.
+		$chunk =~ s/(\A.*(?:\n|\Z))//;
+		$line = $1;
+		if ($line =~ /[ ]\n\Z/) {
+		    # Have a flowed line
+		    $inflow = 1;
+		    if ($infixed) {
+			$new_chunk .= $endfixq;
+			$infixed = 0;
 		    }
+		    if ($nonfixed) {
+			$new_chunk .= $line;
+		    } else {
+			$new_chunk .= '<tt>' . $line . '</tt>';
+		    }
+		    next FLOWED_LINE;
+		}
+		if ($inflow) {
+		    # Last line of flowed text may not have SP CRLF
+		    if ($nonfixed) {
+			$new_chunk .= $line . "<br>";
+		    } else {
+			$new_chunk .= '<tt>' . $line . '</tt>';
+		    }
+		    $inflow = 0;
+		    next FLOWED_LINE;
+		}
+		# Fixed line
+		if (!$infixed) {
+		    # Begin fixed rendering if at start
+		    $new_chunk .= $startfixq . "\n";
+		    $infixed = 1;
+		    $inflow = 0;
+		}
+		if ($maxwidth > 0) {
+		    # Fixed lines should be clipped to specified max.
+		    $line = break_lines($line, $charset, $maxwidth, 1);
+		}
+		if ($nonfixed) {
+		    # Proportional font desired
+		    $line =~ s/(\n)/<br>$1/g;
+		    if ($keepspace) {
+			$line =~ s/^(.*)$/&preserve_space($1)/gem;
+		    }
+		}
+		$new_chunk .= $line;
 
-		} else {
-		    # fixed format
-		    $para =~ s/^(.*)$
-			      /&break_line($1,
-				  $maxwidth+(length($1)-&html_length($1)))
-			      /gemx
-			if $maxwidth > 0;
-		    if ($nonfixed) {
-			$para =~ s/(\n)/<br>$1/g;
-			if ($keepspace) {
-			    $para =~ s/^(.*)$/&preserve_space($1)/gem;
-			}
-			$chunk .= $para;
-		    } else {
-			$chunk .= $startfixq . $para . $endfixq;
-		    }
-		}
+	    } # End: FLOWED_LINE: while()
+
+	    # Make sure to close tags
+	    if ($infixed) {
+		$new_chunk .= $endfixq;
 	    }
 
+	    # Add quote markup
 	    my $newdepth = length($qd)/length('&gt;');
 	    if ($currdepth < $newdepth) {
-		$chunk = $startq x ($newdepth - $currdepth) . $chunk;
+		$new_chunk = $startq x ($newdepth - $currdepth) . $new_chunk;
 	    } elsif ($currdepth > $newdepth) {
-		$chunk = $endq   x ($currdepth - $newdepth) . $chunk;
+		$new_chunk = $endq   x ($currdepth - $newdepth) . $new_chunk;
 	    }
 	    $currdepth = $newdepth;
-	    $ret .= $chunk;
+	    $ret .= $new_chunk;
 	}
 	if ($currdepth > 0) {
 	    $ret .= $endq x $currdepth;
 	}
 
-	## Post-processing cleanup
-	$ret =~ s/<\/pre>\s*<br>\s*((?:<br>\s*)+)/<\/pre>$1/g;
-
 	$$data = $ret;
 
     } elsif ($quote_style == Q_FANCY) {
-	# Fancy code very similiar to flowed code, but simplier.
-	# Some stuff is "duplicated", but this is written to use
-	# ${HQuoteChars}, which would allow for alternate
-	# quote characters beyond '>'.
+	# Fancy quoting supports alternative quote characters besides
+	# '>' as defined by ${HQuoteChars}.
 	my($chunk, $qd, $qd_re, $offset);
 	my $currdepth = 0;
 	my $ret='';
@@ -524,8 +557,39 @@ sub filter {
     }
 
     ## Convert URLs to hyperlinks
-    $$data =~ s@($HUrlExp)@<a $target href="$1">$1</a>@gio
-	unless $nourl;
+    if (!$nourl) {
+	my $nolink = undef;
+	my $link   = undef;
+	if ($args =~ /\bnolink\s*=(\S+)/) {
+	    $nolink = lc(','.$1.',');
+	    $nolink =~ s/['"]//g;
+	}
+	if ($args =~ /\blink\s*=(\S+)/) {
+	    $link = lc(','.$1.',');
+	    $link =~ s/['"]//g;
+	}
+	$$data =~ s{
+	    ($HUrlExp)
+	}{
+	    if (!defined($nolink) && !defined($link)) {
+		join('', '<a ', $target, ' rel="nofollow" href="',
+			 $1, '">', $1, '</a>');
+	    } else {
+		my $url_match = $1;
+		my $scheme;
+		$url_match =~ /^([^:]+)/;
+		$scheme = ',' . lc($1) . ',';
+		if ((defined($nolink) && (index($nolink, $scheme) >= $[)) ||
+		    (defined($link)   && (index($link, $scheme)   <  $[)))
+		{
+		    $url_match;
+		} else {
+		    join('', '<a ', $target, ' rel="nofollow" href="',
+			     $url_match, '">', $url_match, '</a>');
+		}
+	    }
+	}gxeso;
+    }
 
     $$data = ' '  if $$data eq '';
     ($$data);
@@ -565,7 +629,42 @@ sub preserve_space {
 
 ##---------------------------------------------------------------------------##
 
-sub break_line {
+sub break_lines {
+    my $data_in  = shift;
+    my $charset  = shift;
+    my $maxwidth = shift;
+    my $is_html	 = shift; # hack for flowed processing
+    return unless $maxwidth > 0;
+
+    my $data = ref($data_in) ? $data_in : \$data_in;
+    my $do_encode = 0;
+    eval {
+	require Encode;
+	# Only translate to Perl's utf-8 if not an 8-bit charset.
+	# No harm if done for 8-bit, but try to avoid unnecesary
+	# overhead.  Translation done so line breaking is done
+	# on characters, not octets.
+	if ($charset !~ /us-ascii/i &&
+		$charset !~ /8859/i &&
+		!Encode::is_utf8($$data)) {
+	    $$data = Encode::decode($charset, $$data);
+	    $do_encode = 1;
+	}
+    };
+    $$data =~ s{
+	^(.*)$
+    }{
+	_break_line($1, ($is_html ? $maxwidth+(length($1)-html_length($1))
+				  : $maxwidth))
+    }gemx;
+    if ($do_encode) {
+	# Translate back to current encoding.
+	$$data = Encode::encode($charset, $$data);
+    }
+    $$data;
+}
+
+sub _break_line {
     my($str) = shift;
     my($width) = shift;
     my($q, $new) = ('', '');
