@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	$Id: mhutil.pl,v 2.27 2003/01/09 23:42:28 ehood Exp $
+##	$Id: mhutil.pl,v 2.32 2005/07/08 05:27:53 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
@@ -39,6 +39,15 @@ use MHonArc::RFC822;
     'list-unsubscribe' 	=> 1,
 );
 
+## Do not apply ADDRESSMODIFYCODE headerfiels
+%HFieldsAsIsList = (
+    %HFieldsList,
+    'content-disposition' => 1,
+    'content-id'          => 1,
+    'content-type'        => 1,
+    'message-id'          => 1,
+);
+
 ## Header fields that contain addresses
 %HFieldsAddr = (
     'apparently-from'	=> 1,
@@ -47,7 +56,11 @@ use MHonArc::RFC822;
     'cc'		=> 1,
     'dcc'		=> 1,
     'from'		=> 1,
+    'mail-followup-to'	=> 1,
     'mail-reply-to'	=> 1,
+    'notify-bcc'	=> 1,
+    'notify-cc' 	=> 1,
+    'notify-to' 	=> 1,
     'original-bcc'	=> 1,
     'original-cc'	=> 1,
     'original-from'	=> 1,
@@ -454,7 +467,8 @@ sub htmlize_header {
 		foreach $tmp (@array) {
 		    $tmp = $HFieldsList{$key} ? mlist_field_add_links($tmp) :
 						&$MHeadCnvFunc($tmp);
-		    $tmp = field_add_links($key, $tmp, $fields);
+		    $tmp = field_add_links($key, $tmp, $fields)
+			unless $HFieldsAsIsList{$key};
 		    ($tago, $tagc, $ftago, $ftagc) = get_header_tags($key);
 		    $mesg .= join('', $LABELBEG,
 				  $tago, htmlize(ucfirst($key)), $tagc,
@@ -470,7 +484,8 @@ sub htmlize_header {
 		foreach $tmp (@array) {
 		    $tmp = $HFieldsList{$item} ? mlist_field_add_links($tmp) :
 						 &$MHeadCnvFunc($tmp);
-		    $tmp = field_add_links($item, $tmp, $fields);
+		    $tmp = field_add_links($item, $tmp, $fields)
+			unless $HFieldsAsIsList{$key};
 		    ($tago, $tagc, $ftago, $ftagc) = &get_header_tags($item);
 		    $mesg .= join('', $LABELBEG,
 				  $tago, htmlize(ucfirst($item)), $tagc,
@@ -492,7 +507,7 @@ sub mlist_field_add_links {
     my $txt	= shift;
     my $ret	= "";
     local($_);
-    foreach (split(/(<[^>]+>)/, $txt)) {
+    foreach (split(/(<[^<>]+>)/, $txt)) {
 	if (/^<\w+:/) {
 	    chop; substr($_, 0, 1) = "";
 	    $ret .= qq|&lt;<a href="$_">$_</a>&gt;|;
@@ -512,23 +527,20 @@ sub field_add_links {
     my $fields	 = shift;
 
     LBLSW: {
-	if ($HFieldsAddr{$label}) {
-	    if (!$NOMAILTO) {
-		$fld_text =~ s{($HAddrExp)}
-			      {&mailUrl($1, $fields->{'x-mha-message-id'},
-					    $fields->{'x-mha-subject'},
-					    $fields->{'x-mha-from'});
-			      }gexo;
-	    } else {
-		$fld_text =~ s{($HAddrExp)}
-			      {&htmlize(&rewrite_address($1))
-			      }gexo;
-	    }
+	if (!$NONEWS && ($label eq 'newsgroup' || $label eq 'newsgroups')) {
+	    $fld_text = newsurl($fld_text, $fields->{'x-mha-message-id'});
 	    last LBLSW;
 	}
-	if (!$NONEWS && ($label eq 'newsgroup' || $label eq 'newsgroups')) {
-	    $fld_text = newsurl($fld_text);
-	    last LBLSW;
+	if (!$NOMAILTO) {
+	    $fld_text =~ s{($HAddrExp)}
+			  {&mailUrl($1, $fields->{'x-mha-message-id'},
+					$fields->{'x-mha-subject'},
+					$fields->{'x-mha-from'});
+			  }gexo;
+	} else {
+	    $fld_text =~ s{($HAddrExp)}
+			  {&htmlize(&rewrite_address($1))
+			  }gexo;
 	}
 	last LBLSW;
     }
@@ -540,16 +552,23 @@ sub field_add_links {
 ##	Routine to add news links of newsgroups names
 ##
 sub newsurl {
-    my $str = shift;
+    my $str     = shift;
+    my $msgid_u = urlize(shift);
     my $h = "";
 
+    local $_;
     if ($str =~ s/^([^:]*:\s*)//) {
 	$h = $1;
     }
-    $str =~ s/\s//g;			# Strip whitespace
-    my @groups = split(/,/, $str);	# Split groups
-    foreach (@groups) {			# Make hyperlinks
-	s|(.*)|<a href="news:$1">$1</a>|;
+    $str =~ s/[\s<>]//g;
+    my @groups = split(/,/, $str);
+    my $group;
+    foreach $group (@groups) {
+	my $url     = $NewsUrl;
+	my $group_u = urlize($group);
+	$url =~ s/\$NEWSGROUP(?::U)?\$/$group_u/g;
+	$url =~ s/\$MSGID(?::U)?\$/$msgid_u/g;
+	$group = qq{<a href="$url">$group</a>};
     }
     $h . join(', ', @groups);	# Rejoin string
 }
@@ -585,8 +604,7 @@ sub mailUrl {
     $url =~ s/\$FROMADDRNAME\$/$faddrnamel/g;
     $url =~ s/\$FROMADDRDOMAIN\$/$faddrdomainl/g;
     $url =~ s/\$MSGID\$/$msgidl/g;
-    $url =~ s/\$SUBJECT\$/$subjectl/g;
-    $url =~ s/\$SUBJECTNA\$/$subjectl/g;
+    $url =~ s/\$SUBJECT(?:NA)?\$/$subjectl/g;
     $url =~ s/\$TO\$/$to/g;
     $url =~ s/\$TOADDRNAME\$/$toname/g;
     $url =~ s/\$TOADDRDOMAIN\$/$todomain/g;
